@@ -69,6 +69,9 @@ class RedditContentObject(object):
         raise AttributeError('\'%s\' has no attribute \'%s\'' % (type(self),
                                                                  attr))
 
+    def __ne__(self, other):
+        return not (self == other)
+
     def __setattr__(self, name, value):
         if value and name == 'subreddit':
             value = Subreddit(self.reddit_session, value, fetch=False)
@@ -210,6 +213,7 @@ class Messageable(RedditContentObject):
     """Interface for RedditContentObjects that can be messaged."""
     def compose_message(self, subject, message):
         """A deprecated alias for send_message. Do not use."""
+        # Remove around the end of 2012
         warnings.warn('compose_message has been renamed to send_message and '
                       'will be removed in a future version. Please update.',
                       DeprecationWarning)
@@ -250,10 +254,10 @@ class Refreshable(RedditContentObject):
         """
         if isinstance(self, Redditor):
             other = Redditor(self.reddit_session, self.name)
-        elif isinstance(self, Subreddit):
-            other = Subreddit(self.reddit_session, self.display_name)
         elif isinstance(self, Submission):
             other = Submission.get_info(self.reddit_session, self.permalink)
+        elif isinstance(self, Subreddit):
+            other = Subreddit(self.reddit_session, self.display_name)
         self.__dict__ = other.__dict__  # pylint: disable-msg=W0201
 
 
@@ -312,6 +316,10 @@ class Voteable(RedditContentObject):
         url = self.reddit_session.config['vote']
         params = {'id': self.content_id,
                   'dir': six.text_type(direction)}
+        # pylint: disable-msg=W0212
+        urls = [urljoin(self.reddit_session.user._url, 'disliked'),
+                urljoin(self.reddit_session.user._url, 'liked')]
+        _request.evict(urls)
         return self.reddit_session.request_json(url, params)
 
 
@@ -341,15 +349,11 @@ class Comment(Approvable, Deletable, Distinguishable, Editable, Inboxable,
         for reply in self._replies:
             reply._update_submission(submission)
 
-    @property
-    def score(self):
-        """Return the comment's score."""
-        return self.ups - self.downs
-
     @property
     def is_root(self):
         """Indicate if the comment is a top level comment."""
-        return self.parent_id is None
+        return (self.parent_id is None or
+                self.parent_id == self.submission.content_id)
 
     @property
     def permalink(self):
@@ -365,6 +369,11 @@ class Comment(Approvable, Deletable, Distinguishable, Editable, Inboxable,
             self._replies = response[1]['data']['children'][0]._replies
         return self._replies
 
+    @property
+    def score(self):
+        """Return the comment's score."""
+        return self.ups - self.downs
+
     @property
     def submission(self):
         """Return the submission object this comment belongs to."""
@@ -400,7 +409,7 @@ class MoreComments(RedditContentObject):
         self._comments = None
 
     def __unicode__(self):
-        return ('[More Comments: %s]' % ','.join(self.children))
+        return '[More Comments: %d]' % self.count
 
     def _update_submission(self, submission):
         self.submission = submission
@@ -506,15 +515,21 @@ class LoggedInRedditor(Redditor):
         url = self.reddit_session.config['unread']
         return self.reddit_session.get_content(url, limit=limit)
 
+    @require_login
+    def my_contributions(self, limit=0):
+        """Return the subreddits where the logged in user is a contributor."""
+        url = self.reddit_session.config['my_con_reddits']
+        return self.reddit_session.get_content(url, limit=limit)
+
     @require_login
     def my_moderation(self, limit=0):
-        """Return all of the current user's subreddits that they moderate."""
+        """Return the subreddits where the logged in user is a mod."""
         url = self.reddit_session.config['my_mod_reddits']
         return self.reddit_session.get_content(url, limit=limit)
 
     @require_login
     def my_reddits(self, limit=0):
-        """Return all of the current user's subscribed subreddits."""
+        """Return the subreddits that the logged in user is subscribed to."""
         url = self.reddit_session.config['my_reddits']
         return self.reddit_session.get_content(url, limit=limit)
 
@@ -811,10 +826,6 @@ class Subreddit(Messageable, NSFWable, Refreshable):
         """Get banned users for this subreddit."""
         return self.reddit_session.get_banned(self, *args, **kwargs)
 
-    def get_settings(self, *args, **kwargs):
-        """Get the settings for this subreddit."""
-        return self.reddit_session.get_settings(self, *args, **kwargs)
-
     def get_contributors(self, *args, **kwargs):
         """Get contributors for this subreddit."""
         return self.reddit_session.get_contributors(self, *args, **kwargs)
@@ -835,6 +846,10 @@ class Subreddit(Messageable, NSFWable, Refreshable):
         """Get the reported submissions on this subreddit."""
         return self.reddit_session.get_reports(self, *args, **kwargs)
 
+    def get_settings(self, *args, **kwargs):
+        """Get the settings for this subreddit."""
+        return self.reddit_session.get_settings(self, *args, **kwargs)
+
     def get_spam(self, *args, **kwargs):
         """Get the spam-filtered items on this subreddit."""
         return self.reddit_session.get_spam(self, *args, **kwargs)
@@ -884,7 +899,7 @@ class Subreddit(Messageable, NSFWable, Refreshable):
 
 
 class UserList(RedditContentObject):
-    """A class for UserList."""
+    """A list of Redditors. Works just like a regular list."""
     def __init__(self, reddit_session, json_dict=None, fetch=False):
         super(UserList, self).__init__(reddit_session, json_dict, fetch)
 
