@@ -21,6 +21,7 @@
 from __future__ import unicode_literals
 
 import random
+import time
 import unittest
 import uuid
 import warnings
@@ -37,10 +38,10 @@ USER_AGENT = 'PRAW_test_suite'
 def flair_diff(root, other):
     """Function for comparing two flairlists supporting optional arguments."""
     keys = ['user', 'flair_text', 'flair_css_class']
-    root_items = set(tuple(item[key] if key in item and item[key] else '' for
-                           key in keys) for item in root)
-    other_items = set(tuple(item[key] if key in item and item[key] else '' for
-                            key in keys) for item in other)
+    root_items = set(tuple(item[key].lower() if key in item and item[key] else
+                           '' for key in keys) for item in root)
+    other_items = set(tuple(item[key].lower() if key in item and item[key] else
+                            '' for key in keys) for item in other)
     return list(root_items - other_items)
 
 
@@ -49,6 +50,29 @@ class BasicHelper(object):
         self.r = Reddit(USER_AGENT)
         self.sr = 'reddit_api_test'
         self.un = 'PyApiTestUser2'
+        self.other_user_name = 'PyApiTestUser3'
+
+        if self.r.config.is_reddit:
+            self.comment_url = self.url('r/redditdev/comments/dtg4j/')
+            self.link_url = self.url('/r/UCSantaBarbara/comments/m77nc/')
+            self.link_url_link = 'http://imgur.com/Vr8ZZ'
+            self.more_comments_url = self.url('/r/redditdev/comments/dqkfz/')
+            self.other_user_id = '6c1xj'
+        else:
+            self.comment_url = self.url('/r/reddit_test6/comments/y/')
+            self.link_url = self.url('/r/reddit_test6/comments/y/')
+            self.link_url_link = 'http://google.com/?q=29.9093488449'
+            self.more_comments_url = self.url('/r/reddit_test6/comments/y/')
+            self.other_user_id = 'pk'
+
+    def delay(self, amount=None):
+        if amount:
+            time.sleep(amount)
+        elif self.r.config.api_request_delay == 0:
+            time.sleep(0.1)
+
+    def disable_cache(self):
+        self.r.config.cache_timeout = 0
 
     def url(self, path):
         # pylint: disable-msg=W0212
@@ -65,28 +89,28 @@ class BasicTest(unittest.TestCase, BasicHelper):
     def setUp(self):
         self.configure()
 
-        if self.r.config.is_reddit:
-            self.self_url = self.url('/r/programming/comments/bn2wi/')
-        else:
-            self.self_url = self.url('/r/bboe/comments/2z/tasdest/')
-
     def test_comments_contains_no_noncomment_objects(self):
-        if self.r.config.is_reddit:
-            url = self.url('/r/programming/comments/bn2wi/')
-        else:
-            url = self.url('/r/reddit_test9/comments/1a/')
-        comments = self.r.get_submission(url=url).comments
+        comments = self.r.get_submission(url=self.comment_url).comments
         self.assertFalse([item for item in comments if not
                           (isinstance(item, Comment) or
                            isinstance(item, MoreComments))])
 
+    def test_equality(self):
+        subreddit = self.r.get_subreddit(self.sr)
+        same_subreddit = self.r.get_subreddit(self.sr)
+        submission = six_next(subreddit.get_hot())
+        self.assertTrue(subreddit == same_subreddit)
+        self.assertFalse(subreddit != same_subreddit)
+        self.assertFalse(subreddit == submission)
+
     def test_get_all_comments(self):
         num = 50
         self.assertEqual(num, len(list(self.r.get_all_comments(limit=num))))
 
     def test_get_controversial(self):
         num = 50
-        self.assertEqual(num, len(list(self.r.get_controversial(limit=num))))
+        result = self.r.get_controversial(limit=num, url_data={'t': 'all'})
+        self.assertEqual(num, len(list(result)))
 
     def test_get_front_page(self):
         num = 50
@@ -94,7 +118,8 @@ class BasicTest(unittest.TestCase, BasicHelper):
 
     def test_get_new(self):
         num = 50
-        self.assertEqual(num, len(list(self.r.get_new(limit=num))))
+        result = self.r.get_new(limit=num, url_data={'sort': 'new'})
+        self.assertEqual(num, len(list(result)))
 
     def test_get_popular_reddits(self):
         num = 50
@@ -102,38 +127,30 @@ class BasicTest(unittest.TestCase, BasicHelper):
 
     def test_get_top(self):
         num = 50
-        self.assertEqual(num, len(list(self.r.get_top(limit=num))))
+        result = self.r.get_top(limit=num, url_data={'t': 'all'})
+        self.assertEqual(num, len(list(result)))
 
     def test_flair_list(self):
         sub = self.r.get_subreddit('python')
         self.assertTrue(six_next(sub.flair_list()))
 
     def test_info_by_known_url_returns_known_id_link_post(self):
-        if self.r.config.is_reddit:
-            url = 'http://imgur.com/Vr8ZZ'
-            comm = self.url('/r/UCSantaBarbara/comments/m77nc/')
-        else:
-            url = 'http://google.com/?q=82.1753988563'
-            comm = self.url('/r/reddit_test8/comments/2s/')
-        found_links = self.r.info(url)
-        tmp = self.r.get_submission(url=comm)
+        found_links = self.r.info(self.link_url_link)
+        tmp = self.r.get_submission(url=self.link_url)
         self.assertTrue(tmp in found_links)
 
     def test_info_by_self_url_raises_warning(self):
+        url = six_next(self.r.get_new(url_data={'sort': 'new'})).permalink
         with warnings.catch_warnings(record=True) as w:
             warnings.simplefilter('always')
-            self.r.info(self.self_url)
+            self.r.info(url)
             self.assertEqual(len(w), 1)
             self.assertEqual(w[0].category, UserWarning)
             self.assertTrue('self' in text_type(w[0].message))
 
     def test_info_by_url_also_found_by_id(self):
-        if self.r.config.is_reddit:
-            url = 'http://imgur.com/Vr8ZZ'
-        else:
-            url = 'http://google.com/?q=82.1753988563'
         # pylint: disable-msg=E1101
-        found_link = six_next(self.r.info(url))
+        found_link = six_next(self.r.info(self.link_url_link))
         found_by_id = self.r.info(thing_id=found_link.content_id)
         self.assertTrue(found_by_id)
         self.assertTrue(found_link in found_by_id)
@@ -171,13 +188,62 @@ class BasicTest(unittest.TestCase, BasicHelper):
             self.fail('Timeout did not raise the proper exception.')
 
 
+class RefreshableTest(unittest.TestCase, AuthenticatedHelper):
+    def setUp(self):
+        self.configure()
+
+    def test_cache(self):
+        subreddit = self.r.get_subreddit(self.sr)
+        title = 'Test Cache: %s' % uuid.uuid4()
+        body = "BODY"
+        original_listing = list(subreddit.get_new_by_date(limit=5))
+        subreddit.submit(title, body)
+        new_listing = list(subreddit.get_new_by_date(limit=5))
+        self.assertEqual(original_listing, new_listing)
+        self.disable_cache()
+        no_cache_listing = list(subreddit.get_new_by_date(limit=5))
+        self.assertNotEqual(original_listing, no_cache_listing)
+
+    def test_refresh_subreddit(self):
+        self.disable_cache()
+        subreddit = self.r.get_subreddit(self.sr)
+        new_description = 'Description %s' % uuid.uuid4()
+        subreddit.update_settings(public_description=new_description)
+        self.assertNotEqual(new_description, subreddit.public_description)
+        subreddit.refresh()
+        self.assertEqual(new_description, subreddit.public_description)
+
+    def test_refresh_submission(self):
+        self.disable_cache()
+        subreddit = self.r.get_subreddit(self.sr)
+        submission = six_next(subreddit.get_top())
+        same_submission = self.r.get_submission(submission_id=submission.id)
+        if submission.likes:
+            submission.downvote()
+        else:
+            submission.upvote()
+        self.assertEqual(submission.likes, same_submission.likes)
+        submission.refresh()
+        self.assertNotEqual(submission.likes, same_submission.likes)
+
+    def test_users_share_cache(self):
+        subreddit = self.r.get_subreddit(self.sr)
+        title = 'Test User Sharing Of Cache: %s' % uuid.uuid4()
+        body = "BODY"
+        original_listing = list(subreddit.get_new_by_date(limit=5))
+        subreddit.submit(title, body)
+        self.r.login('PyApiTestUser2', '1111')
+        new_user_listing = list(subreddit.get_new_by_date(limit=5))
+        self.assertEqual(original_listing, new_user_listing)
+
+
 class EncodingTest(unittest.TestCase, AuthenticatedHelper):
     def setUp(self):
         self.configure()
 
     def test_author_encoding(self):
         # pylint: disable-msg=E1101
-        a1 = six_next(self.r.get_new()).author
+        a1 = six_next(self.r.get_new(url_data={'sort': 'new'})).author
         a2 = self.r.get_redditor(text_type(a1))
         self.assertEqual(a1, a2)
         s1 = six_next(a1.get_submitted())
@@ -203,11 +269,8 @@ class EncodingTest(unittest.TestCase, AuthenticatedHelper):
 class MoreCommentsTest(unittest.TestCase, AuthenticatedHelper):
     def setUp(self):
         self.configure()
-        if self.r.config.is_reddit:
-            url = self.url('/r/photography/comments/pozpi/')
-        else:
-            url = self.url('/r/reddit_test9/comments/1a/')
-        self.submission = self.r.get_submission(url=url)
+        self.r.config.comment_limit = 10
+        self.submission = self.r.get_submission(url=self.more_comments_url)
 
     def test_all_comments(self):
         c_len = len(self.submission.comments)
@@ -217,17 +280,16 @@ class MoreCommentsTest(unittest.TestCase, AuthenticatedHelper):
 
         # pylint: disable-msg=W0212
         self.assertEqual(len(self.submission._comments_by_id), acf_len)
-        self.assertTrue(c_len <= ac_len < cf_len < acf_len)
+        self.assertTrue(c_len < ac_len)
+        self.assertTrue(c_len < cf_len)
+        self.assertTrue(ac_len < acf_len)
+        self.assertTrue(cf_len < acf_len)
 
 
 class CommentAttributeTest(unittest.TestCase, BasicHelper):
     def setUp(self):
         self.configure()
-        if self.r.config.is_reddit:
-            url = self.url('/r/reddit_api_test/comments/qfk25/')
-        else:
-            url = self.url('/r/reddit_api_test/comments/ao/')
-        self.submission = self.r.get_submission(url=url)
+        self.submission = self.r.get_submission(url=self.comment_url)
 
     def test_all_comments(self):
         self.assertTrue(len(self.submission.all_comments))
@@ -300,7 +362,7 @@ class CommentReplyTest(unittest.TestCase, AuthenticatedHelper):
             if submission.num_comments > 0:
                 found = submission
                 break
-        if not found:
+        else:
             self.fail('Could not find a submission with comments.')
         comment = found.comments[0]
         reply = comment.reply(text)
@@ -416,7 +478,7 @@ class FlairTest(unittest.TestCase, AuthenticatedHelper):
         self.assertEqual(sub.link_flair_text, flair_text)
 
     def test_add_link_flair_to_invalid_subreddit(self):
-        sub = six_next(self.r.get_subreddit('bboe').get_new_by_date())
+        sub = six_next(self.r.get_subreddit('python').get_new_by_date())
         self.assertRaises(HTTPError, self.subreddit.set_flair, sub, 'text')
 
     def test_add_user_flair_by_subreddit_name(self):
@@ -446,22 +508,23 @@ class FlairTest(unittest.TestCase, AuthenticatedHelper):
     def test_flair_csv_and_flair_list(self):
         # Clear all flair
         self.subreddit.clear_all_flair()
+        self.delay(5)  # Wait for flair to clear
         self.assertEqual([], list(self.subreddit.flair_list()))
 
         # Set flair
-        flair_mapping = [{'user':'bboe', 'flair_text':'dev'},
-                         {'user':'PyAPITestUser2', 'flair_css_class':'xx'},
-                         {'user':'PyAPITestUser3', 'flair_text':'AWESOME',
-                          'flair_css_class':'css'}]
+        flair_mapping = [{'user': 'reddit', 'flair_text': 'dev'},
+                         {'user': 'PyAPITestUser2', 'flair_css_class': 'xx'},
+                         {'user': 'PyAPITestUser3', 'flair_text': 'AWESOME',
+                          'flair_css_class': 'css'}]
         self.subreddit.set_flair_csv(flair_mapping)
         self.assertEqual([], flair_diff(flair_mapping,
                                         list(self.subreddit.flair_list())))
 
     def test_flair_csv_many(self):
-        users = ('bboe', 'pyapitestuser2', 'pyapitestuser3')
+        users = ('reddit', 'pyapitestuser2', 'pyapitestuser3')
         flair_text_a = 'Flair: %s' % uuid.uuid4()
         flair_text_b = 'Flair: %s' % uuid.uuid4()
-        flair_mapping = [{'user':'bboe', 'flair_text': flair_text_a}] * 99
+        flair_mapping = [{'user': 'reddit', 'flair_text': flair_text_a}] * 99
         for user in users:
             flair_mapping.append({'user': user, 'flair_text': flair_text_b})
         self.subreddit.set_flair_csv(flair_mapping)
@@ -470,7 +533,7 @@ class FlairTest(unittest.TestCase, AuthenticatedHelper):
             self.assertEqual(flair['flair_text'], flair_text_b)
 
     def test_flair_csv_optional_args(self):
-        flair_mapping = [{'user': 'bboe', 'flair_text': 'bboe'},
+        flair_mapping = [{'user': 'reddit', 'flair_text': 'reddit'},
                          {'user': 'pyapitestuser3', 'flair_css_class': 'blah'}]
         self.subreddit.set_flair_csv(flair_mapping)
 
@@ -532,7 +595,7 @@ class LocalOnlyTest(unittest.TestCase, BasicHelper):
                           'a', 'b', 'c')
 
     def test_send_feedback(self):
-        msg = 'You guys are awesome. (Sent from reddit_api python module).'
+        msg = 'You guys are awesome. (Sent from the PRAW python module).'
         self.r.send_feedback('Bryce Boe', 'foo@foo.com', msg)
 
 
@@ -605,7 +668,7 @@ class MessageTest(unittest.TestCase, AuthenticatedHelper):
             if isinstance(msg, Message) and msg.author == self.r.user:
                 found = msg
                 break
-        if not found:
+        else:
             self.fail('Could not find a self-message in the inbox')
         reply = found.reply(text)
         self.assertEqual(reply.parent_id, found.content_id)
@@ -686,20 +749,17 @@ class ModeratorUserTest(unittest.TestCase, AuthenticatedHelper):
 class RedditorTest(unittest.TestCase, AuthenticatedHelper):
     def setUp(self):
         self.configure()
-        if self.r.config.is_reddit:
-            self.other = {'id': '6c1xj', 'name': 'PyApiTestUser3'}
-        else:
-            self.other = {'id': 'pa', 'name': 'PyApiTestUser3'}
-        self.other_user = self.r.get_redditor(self.other['name'])
+        self.other_user = self.r.get_redditor(self.other_user_name)
 
     def test_disliked(self):
         # Pulls from get_liked. Problem here may come from get_liked
         item = six_next(self.r.user.get_liked())
         item.downvote()
-        self.assertTrue(item in self.r.user.get_disliked())
+        self.delay()  # The queue needs to be processed
+        self.assertFalse(item in list(self.r.user.get_liked()))
 
     def test_get_redditor(self):
-        self.assertEqual(self.other['id'], self.other_user.id)
+        self.assertEqual(self.other_user_id, self.other_user.id)
 
     def test_friend(self):
         self.other_user.friend()
@@ -708,7 +768,8 @@ class RedditorTest(unittest.TestCase, AuthenticatedHelper):
         # Pulls from get_disliked. Problem here may come from get_disliked
         item = six_next(self.r.user.get_disliked())
         item.upvote()
-        self.assertTrue(item in self.r.user.get_liked())
+        self.delay()  # The queue needs to be processed
+        self.assertFalse(item in list(self.r.user.get_disliked()))
 
     def test_unfriend(self):
         self.other_user.unfriend()
@@ -717,36 +778,6 @@ class RedditorTest(unittest.TestCase, AuthenticatedHelper):
         self.assertTrue(isinstance(self.r.user, LoggedInRedditor))
 
 
-class RefreshableTest(unittest.TestCase, AuthenticatedHelper):
-    def setUp(self):
-        self.configure()
-        self.real_timeout = self.r.config.cache_timeout
-        self.r.config.cache_timeout = 0
-
-    def tearDown(self):
-        self.r.config.cache_timeout = self.real_timeout
-
-    def test_refresh_subreddit(self):
-        subreddit = self.r.get_subreddit(self.sr)
-        new_description = 'Description %s' % uuid.uuid4()
-        subreddit.update_settings(public_description=new_description)
-        self.assertNotEqual(subreddit.public_description, new_description)
-        subreddit.refresh()
-        self.assertEqual(subreddit.public_description, new_description)
-
-    def test_refresh_submission(self):
-        subreddit = self.r.get_subreddit(self.sr)
-        submission = six_next(subreddit.get_top())
-        same_submission = self.r.get_submission(submission_id=submission.id)
-        if submission.likes:
-            submission.downvote()
-        else:
-            submission.upvote()
-        self.assertEqual(submission.likes, same_submission.likes)
-        submission.refresh()
-        self.assertNotEqual(submission.likes, same_submission.likes)
-
-
 class SubmissionTest(unittest.TestCase, AuthenticatedHelper):
     def setUp(self):
         self.configure()
@@ -756,7 +787,7 @@ class SubmissionTest(unittest.TestCase, AuthenticatedHelper):
         for submission in self.r.user.get_submitted():
             if submission.likes is False:
                 break
-        if not submission or submission.likes is not False:
+        else:
             self.fail('Could not find a down-voted submission.')
         submission.clear_vote()
         # reload the submission
@@ -775,7 +806,7 @@ class SubmissionTest(unittest.TestCase, AuthenticatedHelper):
         for submission in self.r.user.get_submitted():
             if submission.likes is True:
                 break
-        if not submission or submission.likes is not True:
+        else:
             self.fail('Could not find an up-voted submission.')
         submission.downvote()
         # reload the submission
@@ -791,7 +822,7 @@ class SubmissionTest(unittest.TestCase, AuthenticatedHelper):
         for submission in subreddit.get_new_by_date():
             if not submission.hidden:
                 break
-        if not submission or submission.hidden:
+        else:
             self.fail('Could not find a non-reported submission.')
         submission.report()
         # check if submission was reported
@@ -806,7 +837,7 @@ class SubmissionTest(unittest.TestCase, AuthenticatedHelper):
         for submission in self.r.user.get_submitted():
             if not submission.saved:
                 break
-        if not submission or submission.saved:
+        else:
             self.fail('Could not find unsaved submission.')
         submission.save()
         # reload the submission
@@ -820,7 +851,7 @@ class SubmissionTest(unittest.TestCase, AuthenticatedHelper):
             self.fail('Could not find submission in saved links.')
 
     def test_short_link(self):
-        submission = six_next(self.r.get_new())
+        submission = six_next(self.r.get_new(url_data={'sort': 'new'}))
         if self.r.config.is_reddit:
             self.assertTrue(submission.id in submission.short_link)
         else:
@@ -832,7 +863,7 @@ class SubmissionTest(unittest.TestCase, AuthenticatedHelper):
         for submission in self.r.user.get_submitted():
             if submission.saved:
                 break
-        if not submission or not submission.saved:
+        else:
             self.fail('Could not find saved submission.')
         submission.unsave()
         # reload the submission
@@ -844,7 +875,7 @@ class SubmissionTest(unittest.TestCase, AuthenticatedHelper):
         for submission in self.r.user.get_submitted():
             if submission.likes is None:
                 break
-        if not submission or submission.likes is not None:
+        else:
             self.fail('Could not find a non-voted submission.')
         submission.upvote()
         # reload the submission
@@ -862,7 +893,7 @@ class SubmissionCreateTest(unittest.TestCase, AuthenticatedHelper):
             if not item.is_self:
                 found = item
                 break
-        if not found:
+        else:
             self.fail('Could not find link post')
         self.assertRaises(errors.APIException, self.r.submit, self.sr,
                           found.title, url=found.url)
@@ -888,11 +919,6 @@ class SubmissionEditTest(unittest.TestCase, AuthenticatedHelper):
     def setUp(self):
         self.configure()
         self.subreddit = self.r.get_subreddit(self.sr)
-        self.real_timeout = self.r.config.cache_timeout
-        self.r.config.cache_timeout = 0
-
-    def tearDown(self):
-        self.r.config.cache_timeout = self.real_timeout
 
     def test_edit_link(self):
         found = None
@@ -900,7 +926,7 @@ class SubmissionEditTest(unittest.TestCase, AuthenticatedHelper):
             if not item.is_self:
                 found = item
                 break
-        if not found:
+        else:
             self.fail('Could not find link post')
         self.assertRaises(HTTPError, found.edit, 'text')
 
@@ -910,13 +936,14 @@ class SubmissionEditTest(unittest.TestCase, AuthenticatedHelper):
             if item.is_self:
                 found = item
                 break
-        if not found:
+        else:
             self.fail('Could not find self post')
         new_body = '%s\n\n+Edit Text' % found.selftext
         found = found.edit(new_body)
         self.assertEqual(found.selftext, new_body)
 
     def test_mark_as_nsfw(self):
+        self.disable_cache()
         found = None
         for item in self.subreddit.get_top():
             if not item.over_18:
@@ -929,6 +956,7 @@ class SubmissionEditTest(unittest.TestCase, AuthenticatedHelper):
         self.assertTrue(found.over_18)
 
     def test_unmark_as_nsfw(self):
+        self.disable_cache()
         found = None
         for item in self.subreddit.get_top():
             if item.over_18:
@@ -945,22 +973,10 @@ class SubredditTest(unittest.TestCase, AuthenticatedHelper):
     def setUp(self):
         self.configure()
         self.subreddit = self.r.get_subreddit(self.sr)
-        self.real_timeout = self.r.config.cache_timeout
-        self.r.config.cache_timeout = 0
-
-    def tearDown(self):
-        self.r.config.cache_timeout = self.real_timeout
 
     def test_attribute_error(self):
         self.assertRaises(AttributeError, getattr, self.subreddit, 'foo')
 
-    def test_get_my_moderation(self):
-        for subreddit in self.r.user.my_moderation():
-            if text_type(subreddit) == self.sr:
-                break
-        else:
-            self.fail('Could not find moderated reddit in my_moderation.')
-
     def test_get_modqueue(self):
         mod_submissions = list(self.r.get_subreddit('mod').get_modqueue())
         self.assertTrue(len(mod_submissions) > 0)
@@ -979,12 +995,27 @@ class SubredditTest(unittest.TestCase, AuthenticatedHelper):
         sub = self.r.get_subreddit('{0}+{1}'.format(self.sr, 'test'))
         self.assertRaises(errors.ModeratorRequired, sub.get_modqueue)
 
+    def test_my_contributions(self):
+        for subreddit in self.r.user.my_contributions():
+            if text_type(subreddit) == self.sr:
+                break
+        else:
+            self.fail('Could not find contributed reddit in my_contributions.')
+
+    def test_my_moderation(self):
+        for subreddit in self.r.user.my_moderation():
+            if text_type(subreddit) == self.sr:
+                break
+        else:
+            self.fail('Could not find moderated reddit in my_moderation.')
+
     def test_my_reddits(self):
         for subreddit in self.r.user.my_reddits():
             # pylint: disable-msg=W0212
             self.assertTrue(text_type(subreddit) in subreddit._info_url)
 
     def test_nsfw_subreddit(self):
+        self.disable_cache()
         originally_nsfw = self.subreddit.over18
         if originally_nsfw:
             self.subreddit.unmark_as_nsfw()
